/*
 * Copyright 2006 Vincenzo Nicosia vnicosia@diit.unict.it
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


import java.io.*;
import java.net.*;


class RecvThread extends Thread{
		
		private Socket s;
		public RecvThread (Socket _s){
				super();
				this.s = _s;
				setDaemon(true);
				start();
				
		}
		public void run (){
				try {
						BufferedReader in = 
								new BufferedReader(new InputStreamReader(s.getInputStream()));
						while(true){
								String str = in.readLine();
								if (s.isClosed())
										return;
								if (str != null){
										System.out.println("Recv: " + str);
								}
						}
				}
				catch (Exception e){
						System.out.println("Exception in RecvThread!");
						e.printStackTrace();
						System.exit(1);
				}
		}
}

class SendThread extends Thread {
		
		private Socket s;
		
		public SendThread(Socket _s){
				super();
				this.s = _s;
				start();
		}
		
		public void run (){
				try {
						BufferedReader in = 
								new BufferedReader(new InputStreamReader(System.in));
						BufferedWriter out = 
								new BufferedWriter(new OutputStreamWriter(s.getOutputStream()));
						System.out.print("msg> ");
						String str = in.readLine();
						while(str.compareTo("*") != 0){
								out.write(str);
								out.newLine();
								out.flush();
								System.out.print("msg> ");
								yield();
								str = in.readLine();
						}
				}
				catch (Exception e){
						System.out.println("Exception in SendThread!");
						e.printStackTrace();
						System.exit(1);
				}
		}
}

public class ChatClient{
		
		public static void main (String arg[]){
				
				if (arg.length < 2){
						System.out.println("Usage: ChatClient <host> <port>");
						System.exit(1);
				}		
				
				System.out.println("---- Simple Chat client ----\n\n");
				
				try {
						Socket s = new Socket (arg[0], Integer.parseInt(arg[1]));
						RecvThread rt = new RecvThread (s);
						SendThread st = new SendThread (s);
						st.join();
				}
				catch (Exception e){
						System.out.println("Exception in main");
						e.printStackTrace();
				}
		}
}


