/*
 * Copyright 2006 Vincenzo Nicosia vnicosia@diit.unict.it
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


import java.net.*;
import java.util.*;
import java.io.*;

class MessageBuffer {
		
		private LinkedList buff;

		public MessageBuffer (){
				buff = new LinkedList();
		}
		
		public synchronized void sendMessage (String msg){
				//System.out.println("MessageBuffer -- Receiving message:  " + msg  + this);
				buff.add(msg);
				notify();
		}
		
		public synchronized String recvMessage (){
				try{
						//System.out.println("Waiting for messages..." + this);
						if (buff.size() == 0){
								wait();
						}
						//System.out.println("OK! Message GOT!" + this);
						return (String)buff.removeFirst();
				}
				catch (InterruptedException e){
						//System.err.println("Interruptede wait!");
						e.printStackTrace();
						return null;
				}
		}
}


class RecvThread extends Thread {
		
		private MessageBuffer globalQueue;
		private Socket sock;
		private BufferedReader in;
		public RecvThread(Socket __s, MessageBuffer __queue) throws IOException{
				super();
				this.globalQueue = __queue;
				this.sock = __s;
				in = new BufferedReader(new InputStreamReader(sock.getInputStream()));
				start();
		}
		
		public void run (){
				String str;
				//System.out.println("Connection Thread started!");
				try {
						while(true){
								str = in.readLine();
								if (str != null){
										//System.out.println("Received message: "+str);
										globalQueue.sendMessage(str);
								}
								if (sock.isClosed())
										return;
						}
				}
				catch (IOException e){
						e.printStackTrace();
				}
		}
}

class SendThread extends Thread {
		
		private BufferedWriter out;
		private MessageBuffer outBuffer;
		private Socket s;
		
		public SendThread (Socket _s, MessageBuffer _out) throws IOException{
				super ();
				outBuffer = _out;
				s = _s;
				out = new BufferedWriter(new OutputStreamWriter(s.getOutputStream()));
				start ();
		}
		
		public void run (){
				
				try {
						while(true){
								//System.out.println("SendThread...Waiting for messages!");
								String msg = outBuffer.recvMessage();
								if (s.isClosed())
										return;
								//System.out.println("SendThread: got message: "+msg);
								if (msg != null){
										out.write(msg);
										out.newLine();
										out.flush();
								}
						}
				}
				catch (Exception e){
						e.printStackTrace();
				}
		}
}


class ConnectionBuffer {
		
		private LinkedList tList;
		
		public ConnectionBuffer(){
				this.tList = new LinkedList();
		}
		
		public void addConnection (MessageBuffer m){
				tList.add(m);
		}
		public void removeConnection (){
				// FIXME: Remove a thread looking at specified address!!!
				tList.removeFirst();
		}
		
		public void sendMessage (String msg){
				
				Iterator i = tList.iterator();
				try {
						while (i.hasNext()){
								//System.out.println("Sending message to Connection Thread!!!");
								((MessageBuffer)(i.next())).sendMessage(msg);
								//System.out.println("Message to Connection Thread sent!!!");
						}
				}
				catch(Exception e){
						e.printStackTrace();
				}
		}
}


class SpoolerThread extends Thread{
		
		MessageBuffer buff;
		ConnectionBuffer tBuff;
		
		public SpoolerThread (MessageBuffer __msg, ConnectionBuffer __thr){
				super();
				this.buff = __msg;
				this.tBuff = __thr;
				start();
		}
		
		public void run (){
				String str;
				//System.out.println("Spooler Thread started!!!");
				while(true){
						str = buff.recvMessage();
						//System.out.println("Spooler received message: "+str);
						tBuff.sendMessage(str);
				}
		}
}


class ChatServ {
		
		private static int port = 9999;
		private static MessageBuffer msg;
		private static ConnectionBuffer cnb;
		private static SpoolerThread spool;
		private static ServerSocket ss;
		
		public static void main (String arg[]){
				
				try {
						// - Create messageBuffer, threadBuffer and spooler
						msg = new MessageBuffer();
						cnb = new ConnectionBuffer();
						spool = new SpoolerThread(msg, cnb);
						// - 
						ss = new ServerSocket(port);
						while (true){
								Socket s = ss.accept();
								RecvThread rt = new RecvThread(s, msg);
								MessageBuffer m = new MessageBuffer();
								cnb.addConnection(m);
								SendThread st = new SendThread(s, m);
						}
						
				}
				catch(Exception e){
						e.printStackTrace();
				}
				
		}
}
